"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.SyntaxHighlighterRegistry = exports.SyntaxHighlighter = void 0;
var _core = require("@shikijs/core");
var _engineJavascript = require("@shikijs/engine-javascript");
var _react = _interopRequireDefault(require("react"));
var _reactNative = require("react-native");
var _jsxRuntime = require("react/jsx-runtime");
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
// Singleton registries
let highlighterInstance = null;
const registeredLanguages = new Map();
const registeredThemes = new Map();

// Initialize the highlighter
async function getHighlighter() {
  if (!highlighterInstance) {
    highlighterInstance = await (0, _core.createHighlighterCore)({
      engine: (0, _engineJavascript.createJavaScriptRegexEngine)(),
      themes: [],
      langs: []
    });
  }
  return highlighterInstance;
}

// Helper to get the first valid language registration from an array or single registration
function getLanguageRegistration(lang) {
  if (Array.isArray(lang)) {
    // Take the first valid language registration
    const validLang = lang.find(l => l && typeof l === 'object' && 'name' in l);
    if (!validLang) {
      throw new Error('No valid language registration found in array');
    }
    return validLang;
  }
  return lang;
}

// Public registration API
const SyntaxHighlighterRegistry = exports.SyntaxHighlighterRegistry = {
  registerLanguage: async (name, language) => {
    const highlighter = await getHighlighter();
    const langReg = getLanguageRegistration(language);
    await highlighter.loadLanguage(langReg);
    registeredLanguages.set(name, langReg);
  },
  registerTheme: async (name, theme) => {
    const highlighter = await getHighlighter();
    await highlighter.loadTheme(theme);
    registeredThemes.set(name, theme);
  },
  isLanguageRegistered: name => registeredLanguages.has(name),
  isThemeRegistered: name => registeredThemes.has(name),
  getRegisteredLanguages: () => Array.from(registeredLanguages.keys()),
  getRegisteredThemes: () => Array.from(registeredThemes.keys())
};
const SyntaxHighlighter = ({
  text,
  language,
  theme,
  fontSize = 14,
  textStyle,
  scrollViewProps,
  containerStyle,
  debugMode = false
}) => {
  const [tokens, setTokens] = _react.default.useState([]);
  const [loading, setLoading] = _react.default.useState(true);
  const [error, setError] = _react.default.useState(null);
  _react.default.useEffect(() => {
    const highlight = async () => {
      try {
        setLoading(true);
        const highlighter = await getHighlighter();
        if (debugMode) {
          console.log('Highlighter state:', {
            registeredLanguages: SyntaxHighlighterRegistry.getRegisteredLanguages(),
            registeredThemes: SyntaxHighlighterRegistry.getRegisteredThemes(),
            requestedLanguage: language,
            requestedTheme: theme
          });
        }

        // Verify language and theme are registered
        if (!SyntaxHighlighterRegistry.isLanguageRegistered(language)) {
          throw new Error(`Language '${language}' not registered`);
        }
        const themeName = typeof theme === 'string' ? theme : theme.name;
        if (!themeName) {
          throw new Error('Theme must have a name property');
        }
        const result = highlighter.codeToTokens(text, {
          lang: language,
          theme: themeName,
          includeExplanation: true
        });
        if (debugMode && result.tokens[0]) {
          console.log('First line token analysis:', result.tokens[0].map(token => ({
            content: token.content,
            color: token.color,
            scopes: token.explanation?.[0]?.scopes.map(s => s.scopeName)
          })));
        }
        setTokens(result.tokens);
        setError(null);
      } catch (err) {
        if (debugMode) {
          console.error('Highlighting error:', {
            error: err,
            message: err instanceof Error ? err.message : 'Unknown error',
            state: {
              text: text.substring(0, 100) + (text.length > 100 ? '...' : ''),
              language,
              registeredLanguages: SyntaxHighlighterRegistry.getRegisteredLanguages(),
              registeredThemes: SyntaxHighlighterRegistry.getRegisteredThemes()
            }
          });
        }
        setError(err instanceof Error ? err.message : 'Failed to highlight code');
      } finally {
        setLoading(false);
      }
    };
    highlight();
  }, [text, language, theme, debugMode]);
  const renderToken = (token, index) => {
    let color = token.color;
    if (!color && token.explanation?.[0]?.scopes) {
      for (const scope of token.explanation[0].scopes) {
        if (scope.themeMatches?.[0]?.settings.foreground) {
          color = scope.themeMatches[0].settings.foreground;
          break;
        }
      }
    }
    const themeObj = typeof theme === 'string' ? registeredThemes.get(theme) : theme;
    const defaultColor = themeObj?.colors?.['editor.foreground'] || '#e1e4e8';
    const tokenStyle = {
      fontSize,
      color: color || defaultColor,
      ...(token.fontStyle === 1 && {
        fontStyle: 'italic'
      }),
      ...(token.fontStyle === 2 && {
        fontWeight: 'bold'
      }),
      ...(token.fontStyle === 4 && {
        textDecorationLine: 'underline'
      })
    };
    const style = _reactNative.StyleSheet.flatten([textStyle, tokenStyle]);
    return /*#__PURE__*/(0, _jsxRuntime.jsx)(_reactNative.Text, {
      style: style,
      children: token.content
    }, `${token.offset}-${index}`);
  };
  const renderLine = (lineTokens, index) => /*#__PURE__*/(0, _jsxRuntime.jsx)(_reactNative.View, {
    style: {
      flexDirection: 'row',
      flexWrap: 'wrap'
    },
    children: lineTokens.map((token, tokenIndex) => renderToken(token, tokenIndex))
  }, index);
  const themeObj = typeof theme === 'string' ? registeredThemes.get(theme) : theme;
  const backgroundColor = themeObj?.colors?.['editor.background'];
  if (loading) {
    return /*#__PURE__*/(0, _jsxRuntime.jsx)(_reactNative.View, {
      style: [{
        backgroundColor,
        padding: 10
      }, containerStyle],
      children: /*#__PURE__*/(0, _jsxRuntime.jsx)(_reactNative.ActivityIndicator, {})
    });
  }
  if (error) {
    return /*#__PURE__*/(0, _jsxRuntime.jsx)(_reactNative.View, {
      style: [{
        backgroundColor,
        padding: 10
      }, containerStyle],
      children: /*#__PURE__*/(0, _jsxRuntime.jsxs)(_reactNative.Text, {
        style: {
          color: '#ff6b6b'
        },
        children: ["Error:", error]
      })
    });
  }
  return /*#__PURE__*/(0, _jsxRuntime.jsx)(_reactNative.ScrollView, {
    ...scrollViewProps,
    horizontal: true,
    style: [{
      backgroundColor
    }, scrollViewProps?.style],
    contentContainerStyle: [containerStyle],
    children: /*#__PURE__*/(0, _jsxRuntime.jsx)(_reactNative.View, {
      onStartShouldSetResponder: () => true,
      children: tokens.map((lineTokens, index) => renderLine(lineTokens, index))
    })
  });
};
exports.SyntaxHighlighter = SyntaxHighlighter;
var _default = exports.default = SyntaxHighlighter;
//# sourceMappingURL=SyntaxHighlighter.js.map