import type { LanguageRegistration, ThemedToken, ThemeRegistrationAny } from '@shikijs/types'
import { createHighlighterCore } from '@shikijs/core'
import { createJavaScriptRegexEngine } from '@shikijs/engine-javascript'
import React, { type FunctionComponent } from 'react'
import { ActivityIndicator, ScrollView, type ScrollViewProps, type StyleProp, StyleSheet, Text, type TextStyle, View, type ViewStyle } from 'react-native'

export interface SyntaxHighlighterProps {
  text: string
  language: string
  theme: string | ThemeRegistrationAny
  fontSize?: number
  textStyle?: StyleProp<TextStyle>
  scrollViewProps?: ScrollViewProps
  containerStyle?: StyleProp<ViewStyle>
  debugMode?: boolean
}

// Singleton registries
let highlighterInstance: Awaited<ReturnType<typeof createHighlighterCore>> | null = null
const registeredLanguages = new Map<string, LanguageRegistration>()
const registeredThemes = new Map<string, ThemeRegistrationAny>()

// Initialize the highlighter
async function getHighlighter() {
  if (!highlighterInstance) {
    highlighterInstance = await createHighlighterCore({
      engine: createJavaScriptRegexEngine(),
      themes: [],
      langs: [],
    })
  }
  return highlighterInstance
}

// Helper to get the first valid language registration from an array or single registration
function getLanguageRegistration(lang: LanguageRegistration | LanguageRegistration[]): LanguageRegistration {
  if (Array.isArray(lang)) {
    // Take the first valid language registration
    const validLang = lang.find(l => l && typeof l === 'object' && 'name' in l)
    if (!validLang) {
      throw new Error('No valid language registration found in array')
    }
    return validLang
  }
  return lang
}

// Public registration API
export const SyntaxHighlighterRegistry = {
  registerLanguage: async (name: string, language: LanguageRegistration | LanguageRegistration[]) => {
    const highlighter = await getHighlighter()
    const langReg = getLanguageRegistration(language)
    await highlighter.loadLanguage(langReg)
    registeredLanguages.set(name, langReg)
  },

  registerTheme: async (name: string, theme: ThemeRegistrationAny) => {
    const highlighter = await getHighlighter()
    await highlighter.loadTheme(theme)
    registeredThemes.set(name, theme)
  },

  isLanguageRegistered: (name: string) => registeredLanguages.has(name),
  isThemeRegistered: (name: string) => registeredThemes.has(name),

  getRegisteredLanguages: () => Array.from(registeredLanguages.keys()),
  getRegisteredThemes: () => Array.from(registeredThemes.keys()),
}

export const SyntaxHighlighter: FunctionComponent<SyntaxHighlighterProps> = ({ text, language, theme, fontSize = 14, textStyle, scrollViewProps, containerStyle, debugMode = false }) => {
  const [tokens, setTokens] = React.useState<ThemedToken[][]>([])
  const [loading, setLoading] = React.useState(true)
  const [error, setError] = React.useState<string | null>(null)

  React.useEffect(() => {
    const highlight = async () => {
      try {
        setLoading(true)
        const highlighter = await getHighlighter()

        if (debugMode) {
          console.log('Highlighter state:', {
            registeredLanguages: SyntaxHighlighterRegistry.getRegisteredLanguages(),
            registeredThemes: SyntaxHighlighterRegistry.getRegisteredThemes(),
            requestedLanguage: language,
            requestedTheme: theme,
          })
        }

        // Verify language and theme are registered
        if (!SyntaxHighlighterRegistry.isLanguageRegistered(language)) {
          throw new Error(`Language '${language}' not registered`)
        }

        const themeName = typeof theme === 'string' ? theme : theme.name
        if (!themeName) {
          throw new Error('Theme must have a name property')
        }

        const result = highlighter.codeToTokens(text, {
          lang: language,
          theme: themeName,
          includeExplanation: true,
        })

        if (debugMode && result.tokens[0]) {
          console.log(
            'First line token analysis:',
            result.tokens[0].map(token => ({
              content: token.content,
              color: token.color,
              scopes: token.explanation?.[0]?.scopes.map(s => s.scopeName),
            })),
          )
        }

        setTokens(result.tokens)
        setError(null)
      }
      catch (err) {
        if (debugMode) {
          console.error('Highlighting error:', {
            error: err,
            message: err instanceof Error ? err.message : 'Unknown error',
            state: {
              text: text.substring(0, 100) + (text.length > 100 ? '...' : ''),
              language,
              registeredLanguages: SyntaxHighlighterRegistry.getRegisteredLanguages(),
              registeredThemes: SyntaxHighlighterRegistry.getRegisteredThemes(),
            },
          })
        }
        setError(err instanceof Error ? err.message : 'Failed to highlight code')
      }
      finally {
        setLoading(false)
      }
    }

    highlight()
  }, [text, language, theme, debugMode])

  const renderToken = (token: ThemedToken, index: number) => {
    let color = token.color
    if (!color && token.explanation?.[0]?.scopes) {
      for (const scope of token.explanation[0].scopes) {
        if (scope.themeMatches?.[0]?.settings.foreground) {
          color = scope.themeMatches[0].settings.foreground
          break
        }
      }
    }

    const themeObj = typeof theme === 'string' ? registeredThemes.get(theme) : theme
    const defaultColor = themeObj?.colors?.['editor.foreground'] || '#e1e4e8'

    const tokenStyle: TextStyle = {
      fontSize,
      color: color || defaultColor,
      ...(token.fontStyle === 1 && { fontStyle: 'italic' as const }),
      ...(token.fontStyle === 2 && { fontWeight: 'bold' as const }),
      ...(token.fontStyle === 4 && { textDecorationLine: 'underline' as const }),
    }

    const style = StyleSheet.flatten([textStyle, tokenStyle])

    return (
      <Text key={`${token.offset}-${index}`} style={style}>
        {token.content}
      </Text>
    )
  }

  const renderLine = (lineTokens: ThemedToken[], index: number) => (
    <View key={index} style={{ flexDirection: 'row', flexWrap: 'wrap' }}>
      {lineTokens.map((token, tokenIndex) => renderToken(token, tokenIndex))}
    </View>
  )

  const themeObj = typeof theme === 'string' ? registeredThemes.get(theme) : theme
  const backgroundColor = themeObj?.colors?.['editor.background']

  if (loading) {
    return (
      <View style={[{ backgroundColor, padding: 10 }, containerStyle]}>
        <ActivityIndicator />
      </View>
    )
  }

  if (error) {
    return (
      <View style={[{ backgroundColor, padding: 10 }, containerStyle]}>
        <Text style={{ color: '#ff6b6b' }}>
          Error:
          {error}
        </Text>
      </View>
    )
  }

  return (
    <ScrollView {...scrollViewProps} horizontal style={[{ backgroundColor }, scrollViewProps?.style]} contentContainerStyle={[containerStyle]}>
      <View onStartShouldSetResponder={() => true}>{tokens.map((lineTokens, index) => renderLine(lineTokens, index))}</View>
    </ScrollView>
  )
}

export default SyntaxHighlighter
